"""
    Data Preprocessing
"""

import scipy
import scipy.signal
import numpy as np
from sklearn import preprocessing

import sys
sys.path.append("./")
sys.path.append("..")

from src.models.find_components.Find_Component import FindPCAcomps


# GC Spec data
class SpecPreprocessing():
    def __init__(self, spec_data, spec_raw_pe, gauss_sigma=0.2):

        self.gauss_sigma = gauss_sigma
        self.spec_data = spec_data
        self.spec_raw_pe = spec_raw_pe

        # Output
        self.gaussian = None
        self.spec_data_gc = None
        self.spec_data_gc_fft = None

    def gaussian_convolve(self):
        gx = self.spec_raw_pe[:,:]
        mu = self.spec_raw_pe[0,gx.shape[1]//2]

        self.gaussian = np.exp(-((gx-mu)/self.gauss_sigma)**2/2)/np.sqrt(2*np.pi*self.gauss_sigma**2)
        print(self.spec_data.shape, self.spec_raw_pe.shape, self.gaussian.shape)
        #result = np.convolve(spec_data[0,:], gaussian, mode="full")

        self.spec_data_gc = scipy.signal.fftconvolve(self.spec_data, self.gaussian, mode="same",axes=1)/80
        print(self.spec_data_gc.shape)

        return self.spec_data_gc, self.gaussian

    def make_fft(self):
        
        self.spec_data_gc_fft = np.fft.fft(self.spec_data_gc)

        return self.spec_data_gc_fft
        


class PesPreprocessing():
    def __init__(self, data_train, data_test, n_pca=50):

        self.data_train = data_train
        self.data_test = data_test
        self.n_pca = n_pca

        # Output
        self.pca_model = None
        self.scaler = None
        self.new_train = None
        self.new_test = None
        


    def find_pca(self):
        pes_PCA = FindPCAcomps(
                                data_train=self.data_train, 
                                data_test=self.data_test,
                                n_pca_comps=self.n_pca
                            )

        self.pca_model, self.new_train, self.new_test = pes_PCA.get_pca()
        return self.pca_model, self.new_train, self.new_test

    def stand_scaler(self):

        self.scaler = preprocessing.StandardScaler().fit(self.data_train)
        
        self.new_train = self.scaler.transform(self.data_train)
        self.new_test = self.scaler.transform( self.data_test)

        return self.scaler, self.new_train, self.new_test


    def get(self, data_info):

        if data_info["pes_pca_preprocessing"]:
            self.pca_model, self.new_train, self.new_test = self.find_pca()
            return self.pca_model, self.new_train, self.new_test

        if data_info["standard_scaler"]:
            self.new_train, self.new_test = self.stand_scaler()
            return self.scaler, self.new_train, self.new_test





